/*
 * GkSu -- Gtk+ Frontend to su
 * Copyright (C) 2002 Gustavo Noronha Silva
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h> /* for basename() */
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <getopt.h>
#include <gtk/gtk.h>
#include <locale.h>

#include "../config.h"
#include "defines.h"

#include "gksu.h"

#include "util.h"
#include "gui.h"
#include "xauth.h"
#include "su.h"

struct option long_opts[] = {
    /*
     * { name  has_arg  *flag  val } 
     */
    {"help", no_argument, NULL, 'h'},
    {"login", no_argument, NULL, 'l'},
    {"preserv-env", no_argument, NULL, 'k'},
    {"user", required_argument, NULL, 'u'},
    {"print-pass", no_argument, NULL, 'p'},
    {"message", required_argument, NULL, 'm'},
    {"title", required_argument, NULL, 't'},
    {"icon", required_argument, NULL, 'i'},
    {"disable-grab", no_argument, NULL, 'g'},
    {0, 0, 0, 0}
};

void
help (gchar *cmdname)
{ 
  g_print (_("GKsu version %s\n\n"
	     "Usage: %s [-u <user>] [-k] [-l] <command>\n\n"
	     "\t--user <user>, -u <user>\n"
	     "\t\tCalls <command> as the specified user\n"
	     "\t--message <message>, -m <message>\n"
	     "\t\tReplaces the standard message shown to ask for\n"
	     "\t\tpassword for the argument passed to the option\n"
	     "\t--title <title>, -t <title>\n"
	     "\t\tR eplaces the default title with the argument\n"
	     "\t--icon <icon>, -i <icon>\n"
	     "\t\tReplaces the default window icon with the argument\n"
	     "\t--print-pass, -p\n"
	     "\t\tAsks gksu to print the password to stdout, just\n"
	     "\t\tlike ssh-askpass. Useful to use in scripts with\n"
	     "\t\tprograms that accept receiving the passowrd on\n"
	     "\t\tstdin.\n"
	     "\t--disable-grab, -g\n"
	     "\t\tDisables the \"locking\" of the keyboard, mouse,\n"
	     "\t\tand focus done by the program when asking for\n"
	     "\t\tpassword\n"
	     "\n" 
	     "\t--login, -l\n"
	     "\t\tMakes this a login shell. Beware this may cause\n"
	     "\t\tproblems with the Xauthority magic. Run xhost\n"
	     "\t\tto allow the target user to open windows on your\n"
	     "\t\tdisplay!\n"
	     "\t--preserv-env, -k\n"
	     "\t\tPreserve the current environments, does not set $HOME\n"
	     "\t\tnor $PATH, for example.\n"
	     "\n"
	     "\n"), PACKAGE_VERSION, cmdname);
}

/*
  clears variables used to store password 
*/
void
free_pass (gchar *pass)
{
  if (pass != NULL)
    {
      memset (pass, 0, strlen(pass));
      g_free (pass);
      pass = NULL;
    }
}

int 
main (int argc, char **argv)
{
  gchar *myname;

  GkConf conf;

  int c;

  gboolean print_pass = FALSE;

  conf.xauth = NULL;
  conf.display = NULL;
  conf.dir_template = g_strdup ("/tmp/" PACKAGE_NAME "-XXXXXX");
  conf.dir= NULL;
  conf.user = NULL;
  conf.command = NULL; /* gksu */

  conf.title = NULL;
  conf.message = NULL;
  conf.icon = NULL;

  conf.login_shell = FALSE;
  conf.keep_env = FALSE;
  conf.grab = TRUE;

  setlocale (LC_ALL, "");
  bindtextdomain(PACKAGE_NAME, LOCALEDIR);  
  bind_textdomain_codeset (PACKAGE_NAME, "UTF-8");
  textdomain(PACKAGE_NAME);
  gtk_set_locale();

  gtk_init (&argc, &argv);

  myname = basename (argv[0]);

  while ((c = getopt_long(argc, argv, "?hu:lpm:kt:i:g", long_opts, NULL))
	 != EOF)
    {
      switch (c)
	{
	case 'h':
	  help (argv[0]);
	  exit(0);
	  break;
	case '?':
	  help (argv[0]);
	  exit(0);
	  break;
	case 'u':
	  conf.user = g_strdup (optarg);
	  break;
	case 'l':
	  conf.login_shell = TRUE;
	  break;
	case 'p':
	  print_pass = TRUE;
	  break;
	case 't':
	  conf.title = conv_to_utf8 (optarg);
	  break;
	case 'm':
	  conf.message = conv_to_utf8 (optarg);
	  break;
	case 'i':
	  conf.icon = g_strdup (optarg);
	  break;
	case 'k':
	  conf.keep_env = TRUE;
	  break;
	case 'g':
	  conf.grab = FALSE;
	  break;
	default:
	  fprintf (stderr, _("Using %s as username...\n"), argv[optind]);
	}
    }

  if (conf.title == NULL)
    conf.title = g_strdup (_("Changing user..."));

  if (conf.icon == NULL)
    gtk_window_set_default_icon_from_file (DATA_DIR "/pixmaps/gksu.png", 
					   NULL);
  else
    gtk_window_set_default_icon_from_file (conf.icon, NULL);

  /* if user is not specified, use root */
  if (conf.user == NULL)
    conf.user = g_strdup ("root");

  /* 
     if --print-pass is passed we do not need to care
     about xauth stuff and etc... let's just ask the
     password and print it
  */
  if (print_pass)
    {
      gchar *password;

      if (conf.message == NULL)
	password = ask_password (&conf,
				 _("Please enter %s's password"),
				 conf.user);
      else
	password = ask_password (&conf, conf.message);

      printf ("%s\n", password);
      free_pass (password);

      exit (0);
    }

  /* now we can begin to care about a command */
  if (argc == optind)
    {
      gk_dialog (GTK_MESSAGE_ERROR, _("Missing command to run."));
      exit (1);
    }

  if (prepare_xauth (&conf))
    return 1;
  conf.command = g_strdup (argv[optind]);

  su_do (&conf);
  
  g_free (conf.command);

  g_free (conf.dir_template);
  g_free (conf.title);
  g_free (conf.user);

 return 0;
}
